package com.nisum.ecomcustomer.controller;

import com.nisum.ecomcustomer.model.Customer;
import com.nisum.ecomcustomer.model.CustomerType;
import com.nisum.ecomcustomer.service.CustomerService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.Email;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;
import java.time.LocalDate;
import java.util.List;


@RestController
@Validated
public class CustomerController {

    @Autowired
    CustomerService customerService;

    @PostMapping("/register")
    public ResponseEntity<Customer> registerCustomer(@RequestBody @Valid @NotNull(message = "body can't be empty") Customer customer) {
        return new ResponseEntity<>(customerService.register(customer), HttpStatus.CREATED);
    }

    @PostMapping("/registerAll")
    public ResponseEntity<List<Customer>> registerCustomers(@RequestBody @NotNull @Valid List<Customer> customer) {
        return new ResponseEntity<>(customerService.registerAll(customer), HttpStatus.CREATED);
    }

    @PutMapping("/update")
    public ResponseEntity<Customer> updateCustomer(@RequestBody @NotNull(message = "body can't be empty") @Valid Customer customer) {
        return new ResponseEntity<>(customerService.update(customer), HttpStatus.CREATED);
    }

    @GetMapping("/")
    public ResponseEntity<List<Customer>> getAllCustomers() {
        return ResponseEntity.ok(customerService.getAllCustomers());
    }

    @GetMapping("/id/{id}")
    public ResponseEntity<Customer> getCustomerById(@PathVariable("id") @Pattern(regexp = "^[1-9][\\d]*$", message = "must be a numeric id") String idStr) {
        long id = Long.parseLong(idStr);
        return ResponseEntity.ok(customerService.getCustomerById(id));
    }

    @GetMapping("/email/{email}")
    public ResponseEntity<Customer> getCustomerByEmail(@PathVariable @Email String email) {
        return ResponseEntity.ok(customerService.getCustomerByEmail(email));
    }

    @GetMapping("/fname/{fname}")
    public ResponseEntity<List<Customer>> getCustomersByFirstName(@PathVariable("fname") String fName) {
        return ResponseEntity.ok(customerService.getCustomerByFirstName(fName));
    }

    @GetMapping("/lname/{lname}")
    public ResponseEntity<List<Customer>> getCustomersByLastName(@PathVariable("lname") String lName) {
        return ResponseEntity.ok(customerService.getCustomerByLastName(lName));
    }

    @GetMapping("/type/{type}")
    public ResponseEntity<List<Customer>> getCustomersByCustomerType(@PathVariable("type") int type) {
        CustomerType customerType = CustomerType.of(type);
        return ResponseEntity.ok(customerService.getCustomersByCustomerType(customerType));
    }

    @GetMapping("/registered/on")
    public ResponseEntity<List<Customer>> getRegisteredCustomersOnDate(@RequestParam("onDate") @DateTimeFormat(iso = DateTimeFormat.ISO.DATE) LocalDate onDate) {
        return ResponseEntity.ok(customerService.getRegisteredCustomersOnDate(onDate));
    }

    @GetMapping("/registered/between")
    public ResponseEntity<List<Customer>> getRegisteredCustomersBetweenDates(@RequestParam("fromDate") @DateTimeFormat(iso = DateTimeFormat.ISO.DATE) LocalDate fromDate,
                                                                             @RequestParam("toDate") @DateTimeFormat(iso = DateTimeFormat.ISO.DATE) LocalDate toDate) {
        return ResponseEntity.ok(customerService.getRegisteredCustomersBetweenDates(fromDate, toDate));
    }

    @DeleteMapping("/id/{id}")
    public ResponseEntity<Void> deleteById(@PathVariable("id") @Pattern(regexp = "^[1-9][\\d]*$", message = "must be a numeric id") String idStr) {
        long id = Long.parseLong(idStr);
        customerService.deleteById(id);
        return ResponseEntity.noContent().build();
    }
}
