package com.nisum.ecomcustomer.controller;

import com.nisum.ecomcustomer.model.Address;
import com.nisum.ecomcustomer.model.AddressType;
import com.nisum.ecomcustomer.model.Customer;
import com.nisum.ecomcustomer.model.CustomerType;
import com.nisum.ecomcustomer.repository.CustomerRepository;
import com.nisum.ecomcustomer.repository.DbSequenceRepository;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestInstance;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.web.server.LocalServerPort;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.test.context.ActiveProfiles;

import java.time.LocalDate;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

import static org.junit.jupiter.api.Assertions.*;
import static org.junit.jupiter.api.TestInstance.Lifecycle;
import static org.springframework.boot.test.context.SpringBootTest.WebEnvironment;

@SpringBootTest(webEnvironment = WebEnvironment.DEFINED_PORT)
@TestInstance(Lifecycle.PER_CLASS)
@ActiveProfiles("test")
public class CustomerControllerIntegrationTest {

    @LocalServerPort
    private int port;

    private String url;

    @Autowired
    TestRestTemplate testRestTemplate;

    @Autowired
    CustomerRepository customerRepository;

    @Autowired
    DbSequenceRepository sequenceRepository;

    @BeforeAll
    public void setUp() {
        initializeDb();

        url = "http://localhost:" + port;
    }

    @Test
    void registerCustomer_returnsCustomer_201() {
        Customer requestObject = getCustomerRequestObject();
        requestObject.setEmail("ashokk101@nisum.com");

        ResponseEntity<Customer> customerResponseEntity = testRestTemplate.postForEntity(url + "/register", requestObject, Customer.class);
        Customer customer = customerResponseEntity.getBody();

        assertEquals(HttpStatus.CREATED, customerResponseEntity.getStatusCode());
        assertNotNull(customer);
        assertNotNull(customer.getId());
        assertEquals(LocalDate.now(), customer.getCreatedDate());
    }

    @Test
    void registerCustomer_withExistingEmail_throwsEmailAlreadyRegisteredException_409() {
        Customer requestObject = getCustomerRequestObject();
        requestObject.setEmail("ashokk101@nisum.com");

        ResponseEntity<Customer> customerResponseEntity = testRestTemplate.postForEntity(url + "/register", requestObject, Customer.class);

        assertEquals(HttpStatus.CONFLICT, customerResponseEntity.getStatusCode());
    }

    @Test
    void registerCustomer_withMultipleDefaultShippingAddresses_throwsMethodArgumentNotValidException_404() {
        Customer requestObject = getCustomerRequestObject();
        requestObject.setEmail("ashokk101@nisum.com");
        requestObject.getAddresses().forEach(address -> address.setDefault(true));

        ResponseEntity<Customer> customerResponseEntity = testRestTemplate.postForEntity(url + "/register", requestObject, Customer.class);

        assertEquals(HttpStatus.BAD_REQUEST, customerResponseEntity.getStatusCode());
    }

    @Test
    void registerCustomer_withMultipleBillingAddresses_throwsMethodArgumentNotValidException_404() {
        Customer requestObject = getCustomerRequestObject();
        requestObject.setEmail("ashokk101@nisum.com");
        requestObject.getAddresses().forEach(address -> address.setAddressType(AddressType.BILLING));

        ResponseEntity<Customer> customerResponseEntity = testRestTemplate.postForEntity(url + "/register", requestObject, Customer.class);

        assertEquals(HttpStatus.BAD_REQUEST, customerResponseEntity.getStatusCode());
    }

    @Test
    void updateCustomer_returnsCustomer_201() {
        Customer requestObject = customerRepository.findById(1L).get();
        requestObject.setCustomerType(CustomerType.REGULAR);

        HttpEntity<Customer> httpEntity = new HttpEntity<>(requestObject);
        ResponseEntity<Customer> customerUpdatedResponseEntity = testRestTemplate.exchange(url + "/update", HttpMethod.PUT, httpEntity, Customer.class, new HashMap<>());
        Customer updatedCustomer = customerUpdatedResponseEntity.getBody();

        assertEquals(HttpStatus.CREATED, customerUpdatedResponseEntity.getStatusCode());
        assertNotNull(updatedCustomer);
        assertEquals(1L, updatedCustomer.getId());
        assertEquals(CustomerType.REGULAR, updatedCustomer.getCustomerType());
    }

    @Test
    void updateCustomer_throwsCustomerNotFoundException_404() {
        Customer requestObject = getCustomerRequestObject();
        requestObject.setId(2000L);
        requestObject.setCustomerType(CustomerType.REGULAR);

        HttpEntity<Customer> httpEntity = new HttpEntity<>(requestObject);
        ResponseEntity<Customer> customerUpdatedResponseEntity = testRestTemplate.exchange(url + "/update", HttpMethod.PUT, httpEntity, Customer.class, new HashMap<>());

        assertEquals(HttpStatus.NOT_FOUND, customerUpdatedResponseEntity.getStatusCode());
    }

    @Test
    public void getById_returnsCustomer_200() {
        Long id = 1L;
        ResponseEntity<Customer> customerResponseEntity = testRestTemplate.getForEntity(url + "/id/" + id, Customer.class);

        assertEquals(HttpStatus.OK, customerResponseEntity.getStatusCode());
        Customer customerResponse = customerResponseEntity.getBody();
        assertNotNull(customerResponse);
        assertEquals(id, customerResponse.getId());
        assertEquals("ashokk1@nisum.com", customerResponse.getEmail());
    }

    @Test
    public void getById_throwsCustomerNotFoundException_404() {
        Long id = 2000L;
        ResponseEntity<Customer> customerResponseEntity = testRestTemplate.getForEntity(url + "/id/" + id, Customer.class);

        assertEquals(HttpStatus.NOT_FOUND, customerResponseEntity.getStatusCode());
    }

    @Test
    public void getByEmail_returnsCustomer_200() {
        String email = "ashokk1@nisum.com";
        ResponseEntity<Customer> customerResponseEntity = testRestTemplate.getForEntity(url + "/email/" + email, Customer.class);

        assertEquals(HttpStatus.OK, customerResponseEntity.getStatusCode());
        Customer customerResponse = customerResponseEntity.getBody();
        assertNotNull(customerResponse);
        assertEquals(email, customerResponse.getEmail());
    }

    @Test
    public void getByEmail_throwsCustomerNotFoundException_404() {
        String email = "ashokk_random@nisum.com";
        ResponseEntity<Customer> customerResponseEntity = testRestTemplate.getForEntity(url + "/email/" + email, Customer.class);

        assertEquals(HttpStatus.NOT_FOUND, customerResponseEntity.getStatusCode());
    }

    @Test
    public void getCustomerByCustomerType_returnsCustomers_200() {
        int loyalValue = CustomerType.LOYAL.getValue();
        ResponseEntity<List<Customer>> customersListResponseEntity = testRestTemplate.exchange("/type/" + loyalValue, HttpMethod.GET, null, new ParameterizedTypeReference<List<Customer>>() {
        });

        assertEquals(HttpStatus.OK, customersListResponseEntity.getStatusCode());
        List<Customer> loyalCustomers = customersListResponseEntity.getBody();
        assertNotNull(loyalCustomers);
        assertNotEquals(0, loyalCustomers.size());
        loyalCustomers.forEach(customer -> assertEquals(loyalValue, customer.getCustomerType().getValue()));
    }

    @Test
    public void getRegisteredCustomerOnDate_200() {
        LocalDate today = LocalDate.now();
        ResponseEntity<List<Customer>> customersRegisteredResponseEntity = testRestTemplate.exchange("/registered/on?onDate={onDate}", HttpMethod.GET, null, new ParameterizedTypeReference<List<Customer>>() {
        }, today);

        List<Customer> customersRegistered = customersRegisteredResponseEntity.getBody();
        assertNotNull(customersRegistered);
        assertNotEquals(0, customersRegistered.size());
        customersRegistered.forEach(customer -> assertEquals(today, customer.getCreatedDate()));
    }


    private void initializeDb() {
        customerRepository.deleteAll();
        sequenceRepository.deleteAll();
        Customer customer1 = getCustomerRequestObject();
        customer1.setEmail("ashokk1@nisum.com");
        Customer customer2 = getCustomerRequestObject();
        customer2.setEmail("ashokk2@nisum.com");
        customer2.setCustomerType(CustomerType.REGULAR);
        Customer customer3 = getCustomerRequestObject();
        customer3.setEmail("ashokk3@nisum.com");
        customer3.setCustomerType(CustomerType.LOYAL);
        Customer customer4 = getCustomerRequestObject();
        customer4.setEmail("ashokk4@nisum.com");
        Customer customer5 = getCustomerRequestObject();
        customer5.setEmail("ashokk5@nisum.com");
        List<Customer> customers = Arrays.asList(customer1, customer2, customer3, customer4, customer5);
        customerRepository.saveAll(customers);
    }

    private Customer getCustomerRequestObject() {
        Customer customer = new Customer();
        customer.setId(null);
        customer.setFirstName("Ashok");
        customer.setLastName("Kumar");
        customer.setEmail("ashokk@nisum.com");
        customer.setDayPhone("8989898989");
        customer.setCustomerType(CustomerType.LOYAL);


        Address billingAddress = new Address();
        billingAddress.setAddressLine1("Srinagar colony");
        billingAddress.setAddressLine2("Ameerpet");
        billingAddress.setAddressType(AddressType.BILLING);
        billingAddress.setDefault(true);
        billingAddress.setZipCode("500000");
        billingAddress.setCity("Hyderabad");
        billingAddress.setState("TS");
        billingAddress.setCountry("IND");

        Address shippingAddress1 = new Address();
        shippingAddress1.setAddressLine1("Jawaharnagar colony");
        shippingAddress1.setAddressLine2("Ameerpet");
        shippingAddress1.setAddressType(AddressType.SHIPPING);
        shippingAddress1.setDefault(true);
        shippingAddress1.setZipCode("500001");
        shippingAddress1.setCity("Hyderabad");
        shippingAddress1.setState("TS");
        shippingAddress1.setCountry("IND");

        Address shippingAddress2 = new Address();
        shippingAddress2.setAddressLine1("Krishnanagar colony");
        shippingAddress2.setAddressLine2("Ameerpet");
        shippingAddress2.setAddressType(AddressType.SHIPPING);
        shippingAddress2.setDefault(false);
        shippingAddress2.setZipCode("500002");
        shippingAddress2.setCity("Hyderabad");
        shippingAddress2.setState("TS");
        shippingAddress2.setCountry("IND");

        List<Address> addresses = Arrays.asList(billingAddress, shippingAddress1, shippingAddress2);
        customer.setAddresses(addresses);

        return customer;
    }

    private List<Customer> getCustomerRequestObjectsList() {
        Customer customer1 = getCustomerRequestObject();
        Customer customer2 = getCustomerRequestObject();
        customer2.setEmail("anandk@nisum.com");
        return Arrays.asList(customer1, customer2);
    }

}
