package com.nisum.ecomcustomer.service.impl;

import com.nisum.ecomcustomer.exceptions.CustomerNotFoundException;
import com.nisum.ecomcustomer.exceptions.EmailAlreadyRegisteredException;
import com.nisum.ecomcustomer.model.Address;
import com.nisum.ecomcustomer.model.AddressType;
import com.nisum.ecomcustomer.model.Customer;
import com.nisum.ecomcustomer.model.CustomerType;
import com.nisum.ecomcustomer.repository.CustomerRepository;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.context.ActiveProfiles;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.anyList;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.when;

@SpringBootTest
@ActiveProfiles("test")
class CustomerServiceImplTest {

    @Autowired
    CustomerServiceImpl customerService;

    @MockBean
    CustomerRepository customerRepository;

    @Test
    void register_throwsEmailAlreadyRegisteredException() {
        String email = "ashokk@nisum.com";
        Customer customerRequestToRegister = getCustomerRequestObject();
        when(customerRepository.existsByEmail(email)).thenReturn(true);
        assertThrows(EmailAlreadyRegisteredException.class, () -> customerService.register(customerRequestToRegister));
    }

    @Test
    void register_success() {
        String email = "ashokk@nisum.com";
        Customer customerRequestToRegister = getCustomerRequestObject();
        Customer customerResponseFromRepo = getCustomerResponseObject();
        when(customerRepository.existsByEmail(email)).thenReturn(false);
        when(customerRepository.save(customerRequestToRegister)).thenReturn(customerResponseFromRepo);
        Customer customerResponse = customerService.register(customerRequestToRegister);
        assertNull(customerRequestToRegister.getId());
        assertNotNull(customerResponse.getId());
    }

    @Test
    void registerAll() {
        List<Customer> customersToSave = getCustomerRequestObjectsList();
        List<Customer> customersSavedResponse = getCustomerResponseObjectsList();

        when(customerRepository.existsByEmail(anyString())).thenReturn(false);
        when(customerRepository.saveAll(customersToSave)).thenReturn(customersSavedResponse);
        List<Customer> customersSaved = customerService.registerAll(customersToSave);
        assertNotNull(customersSaved);
        customersSaved.forEach(customer -> assertNotNull(customer.getId()));
    }

    @Test
    void registerAll_throwsEmailAlreadyExistsException() {
        List<Customer> customersToSave = getCustomerRequestObjectsList();
        when(customerRepository.existsByEmailIn(anyList())).thenReturn(true);
        assertThrows(EmailAlreadyRegisteredException.class, () -> customerService.registerAll(customersToSave));
    }

    @Test
    void getCustomerById_success() {
        Customer responseObject = getCustomerResponseObject();
        when(customerRepository.findById(1L)).thenReturn(Optional.of(responseObject));
        Customer customer = customerService.getCustomerById(1L);
        assertEquals(responseObject.getId(), customer.getId());
    }

    @Test
    void getCustomerById_throwsCustomerNotFoundException() {
        when(customerRepository.findById(1L)).thenReturn(Optional.empty());
        assertThrows(CustomerNotFoundException.class, () -> customerService.getCustomerById(1L));
    }


    private Customer getCustomerRequestObject() {
        Customer customer = new Customer();
        customer.setId(null);
        customer.setFirstName("Ashok");
        customer.setLastName("Katam");
        customer.setEmail("ashokk@nisum.com");
        customer.setDayPhone("8989898989");
        customer.setCustomerType(CustomerType.LOYAL);


        Address billingAddress = new Address();
        billingAddress.setAddressLine1("Srinagar colony");
        billingAddress.setAddressLine2("Ameerpet");
        billingAddress.setAddressType(AddressType.BILLING);
        billingAddress.setDefault(true);
        billingAddress.setZipCode("500000");
        billingAddress.setCity("Hyderabad");
        billingAddress.setState("TS");
        billingAddress.setCountry("IND");

        Address shippingAddress1 = new Address();
        shippingAddress1.setAddressLine1("Jawaharnagar colony");
        shippingAddress1.setAddressLine2("Ameerpet");
        shippingAddress1.setAddressType(AddressType.SHIPPING);
        shippingAddress1.setDefault(true);
        shippingAddress1.setZipCode("500001");
        shippingAddress1.setCity("Hyderabad");
        shippingAddress1.setState("TS");
        shippingAddress1.setCountry("IND");

        Address shippingAddress2 = new Address();
        shippingAddress2.setAddressLine1("Krishnanagar colony");
        shippingAddress2.setAddressLine2("Ameerpet");
        shippingAddress2.setAddressType(AddressType.SHIPPING);
        shippingAddress2.setDefault(false);
        shippingAddress2.setZipCode("500002");
        shippingAddress2.setCity("Hyderabad");
        shippingAddress2.setState("TS");
        shippingAddress2.setCountry("IND");

        List<Address> addresses = Arrays.asList(billingAddress, shippingAddress1, shippingAddress2);
        customer.setAddresses(addresses);

        return customer;
    }

    private Customer getCustomerResponseObject() {
        Customer customer = getCustomerRequestObject();
        customer.setId(1L);
        customer.setEmail(customer.getEmail().toLowerCase());
        customer.setCreatedDate(LocalDate.now());
        customer.setModifiedDate(LocalDateTime.now().plusSeconds(2));
        return customer;
    }

    private List<Customer> getCustomerRequestObjectsList() {
        Customer customer1 = getCustomerRequestObject();
        Customer customer2 = getCustomerRequestObject();
        customer2.setEmail("anandk@nisum.com");
        customer2.setFirstName("Anand");
        customer2.setLastName("Kumar");
        Customer customer3 = getCustomerRequestObject();
        customer3.setEmail("anandk1@nisum.com");
        customer3.setCustomerType(CustomerType.REGULAR);
        customer3.setFirstName("Anand");
        customer3.setLastName("Katam");

        return Arrays.asList(customer1, customer2, customer3);
    }

    private List<Customer> getCustomerResponseObjectsList() {
        List<Customer> customers = getCustomerRequestObjectsList();
        Long id = 1L;
        for (Customer customer : customers) {
            customer.setId(id++);
            customer.setEmail(customer.getEmail().toLowerCase());
            customer.setCreatedDate(LocalDate.now());
            customer.setModifiedDate(LocalDateTime.now());
        }
        return customers;
    }
}